#include "TBeamSimData.hh"

#include "TFile.h"
#include "TTree.h"
#include "TH1.h"
#include "TH2.h"
#include "TGenPhaseSpace.h"
#include "TRandom3.h"
#include "TLorentzVector.h"
#include "TVector3.h"
#include "TString.h"
#include "TNamed.h"

#include "TanaSAMURAIParameters.hh"
#include "TanaCalibCoin.hh"
#include "TanaRecoBeam.hh"
#include "TanaCalibBDC.hh"
#include "TanaCalibFDC1.hh"
#include "TanaCalibFDC2.hh"
#include "TanaCalibHOD.hh"
#include "TanaHODPla.hh"
#include "TanaParticle.hh"
#include "TanaDC.hh"
#include "TanaKinematics.hh"

#include "TArtEventStore.hh"
#include "TArtCounter.hh"

#include <iostream>
#include <sstream>
#include <fstream>

void GenInputFromRIDF(TString runname="29f_emp")
{
  const double AMU = 931.494043;//MeV
  const double c_light=299.792458;//mm/ns

  TanaSAMURAIParameters *prm = TanaSAMURAIParameters::Instance();

  Int_t A, Z, nRun;
  Double_t M, fac, Zgate;
  if (runname=="29ne_emp"){
//  // 29Ne 240MeV/u
    A = 29;
    Z = 10;
    M=27026.3;//MeV
    nRun = 466;// 29Ne
    fac = 0.1;
    Zgate = 10;
    prm->LoadParameterList("db/List_29Ne_Emp_29Ne.csv");
  }else if (runname=="29f_emp"){
  // 29F 
    A = 29;
    Z = 9;
    M=27049;//MeV
    nRun = 486;// 29F+Emp
    fac = 1;
    Zgate = 9;
    prm->LoadParameterList("db/List_29F_Emp_29F.csv");
  }else{
    cout<<"Unknown run name :"<<runname.Data()<<endl;
    return;
  }

  TanaCalibCoin *calibcoin = new TanaCalibCoin();
  TanaRecoBeam *recobeam = new TanaRecoBeam();
  TanaCalibBDC *calibbdc1 = recobeam->GetCalibBDC1();
  TanaCalibBDC *calibbdc2 = recobeam->GetCalibBDC2();
  TanaCalibFDC1 *calibfdc1 = new TanaCalibFDC1();
  TanaCalibFDC2 *calibfdc2 = new TanaCalibFDC2();
  TanaCalibHOD *calibhod = new TanaCalibHOD();
  TClonesArray *hod_array = calibhod->GetHODPlaArray();

  TArtEventStore *estore = new TArtEventStore;
  TString ridffilename = Form("/home/kondo/exp/samurai21/anaroot/ridf/sdaq02/run%04d.ridf.gz",nRun);
  estore->Open(ridffilename.Data());
  TArtCounter *counter = TArtCounter::SetCounterFromRIDF(ridffilename.Data());
  counter->SetCounter(counter->GetMaxNum()*fac);

  //-----------------------------------------
  TFile *file = new TFile(Form("root/brho_func/input_run%04d.root",nRun),"recreate");

  TH1* hcoin   = new TH1I("hcoin","COIN",8,0.5,8.5);
  TH1* hcoin_g = new TH1I("hcoin_g","COIN DSB",8,0.5,8.5);

  TH1* haoqz   = new TH2D("haoqz","AoQ Z",400,2.0,3.5, 300,0,15);
  TH1* haoqz_g = new TH2D("haoqz_g","AoQ Z Gated",400,2.0,3.5, 300,0,15);

  TH1* hidz   = new TH2D("hidz","HOD ID Z",24,0.5,24.5, 200,0,15);
  TH1* hidz_g = new TH2D("hidz_g","HOD ID Z Gated",24,0.5,24.5, 200,0,15);

  TTree *tree = new TTree("tree_input","Input tree for trajectories");
  gBeamSimDataArray = new TBeamSimDataArray();
  tree->Branch("TBeamSimData", gBeamSimDataArray);
  Double_t bdc1x,bdc1y;
  Double_t bdc2x,bdc2y;
  Double_t tgtx,tgty,tgta,tgtb;
  Double_t fdc1x,fdc1y;
  Double_t fdc1a,fdc1b;
  Double_t fdc2x,fdc2y;
  Double_t fdc2a,fdc2b;
  Double_t tof713;
  Double_t brho;
  tree->Branch("tof713",&tof713,"tof713/D");
  tree->Branch("brho",&brho,"brho/D");

  tree->Branch("bdc1x",&bdc1x,"bdc1x/D");
  tree->Branch("bdc1y",&bdc1y,"bdc1y/D");
  tree->Branch("bdc2x",&bdc2x,"bdc2x/D");
  tree->Branch("bdc2y",&bdc2y,"bdc2y/D");
  tree->Branch("tgtx",&tgtx,"tgtx/D");
  tree->Branch("tgty",&tgty,"tgty/D");
  tree->Branch("tgta",&tgta,"tgta/D");
  tree->Branch("tgtb",&tgtb,"tgtb/D");
  tree->Branch("fdc1x",&fdc1x,"fdc1x/D");
  tree->Branch("fdc1a",&fdc1a,"fdc1a/D");
  tree->Branch("fdc1y",&fdc1y,"fdc1y/D");
  tree->Branch("fdc1b",&fdc1b,"fdc1b/D");
  tree->Branch("fdc2x",&fdc2x,"fdc2x/D");
  tree->Branch("fdc2a",&fdc2a,"fdc2a/D");
  tree->Branch("fdc2y",&fdc2y,"fdc2y/D");
  tree->Branch("fdc2b",&fdc2b,"fdc2b/D");

  TNamed *header = new TNamed("header","");
  file->Add(header);

  stringstream sheader;
  sheader<<endl;
  sheader<<" A="<<A
	 <<" Z="<<Z
	 <<endl;

  for (int ieve=0;ieve<counter->GetMaxNum();ieve++){
    estore->GetNextEvent();
    counter->Draw();

    gBeamSimDataArray->clear();
    bdc1x=-9999;bdc1y=-9999;
    bdc2x=-9999;bdc2y=-9999;
    tgtx=-9999;tgty=-9999;tgta=-9999;tgtb=-9999;
    fdc1x=-9999;fdc1y=-9999;
    fdc1a=-9999;fdc1b=-9999;
    fdc2x=-9999;fdc2y=-9999;
    fdc2a=-9999;fdc2b=-9999;

    //----------------------------------------------------------------
    // Beam
    calibcoin->ClearData();
    calibcoin->ReconstructData();
    for (int id=1;id<8;++id)  if (calibcoin->IsChTrue(id)) hcoin->Fill(id);
    if (!calibcoin->IsChTrue(1)) {estore->ClearData();continue;}
    for (int id=1;id<8;++id)  if (calibcoin->IsChTrue(id)) hcoin_g->Fill(id);

    //----------------------------------------------------------------
    // Beam
    recobeam->ClearData();
    recobeam->ReconstructData();
    TanaParticle *beam = recobeam->GetBeamParticle();
    if (beam==0) {estore->ClearData();continue;}

    Double_t Zbeam=beam->GetZ();
    Double_t AoQbeam = beam->GetAoQ();
    haoqz->Fill(AoQbeam,Zbeam);
    if (!recobeam->OK_PID()){estore->ClearData();continue;}
    haoqz_g->Fill(AoQbeam,Zbeam);
    //-------------------------------------------------------------------
    // HOD
    calibhod->ClearData();
    calibhod->ReconstructData();
    TanaHODPla *pla_max = 0;
    Double_t Zmax=0;
    for (int ipla=0;ipla<hod_array->GetEntries();++ipla){
      TanaHODPla *pla = (TanaHODPla*)hod_array->At(ipla);
      Double_t Ztmp = pla->GetZ();
      if (pla->GetID()==11) pla->SetZ(Ztmp*0.97);
      if (Zmax < pla->GetZ() ) {
	Zmax = pla->GetZ();
	pla_max = pla;
      }
    }
    if (pla_max==0) {estore->ClearData();continue;}
    hidz->Fill(pla_max->GetID(), pla_max->GetZ());
    if (fabs(Zmax-Zgate)>0.5) {estore->ClearData();continue;}
    hidz_g->Fill(pla_max->GetID(), pla_max->GetZ());
  
    //-------------------------------------------------------------------
    calibbdc1->ReconstructData();
    TanaDC *bdc1 = calibbdc1->GetDC();
    if (!bdc1->GetOKTrack()){estore->ClearData();continue;}

    calibbdc2->ReconstructData();
    TanaDC *bdc2 = calibbdc2->GetDC();
    if (!bdc2->GetOKTrack()){estore->ClearData();continue;}

    calibfdc1->ClearData();
    calibfdc1->ReconstructData();
    TanaDC *fdc1 = calibfdc1->GetDC();
    if (!fdc1->GetOKTrack()){estore->ClearData();continue;}

    calibfdc2->ClearData();
    calibfdc2->ReconstructData();
    TanaDC *fdc2 = calibfdc2->GetDC();
    if (!fdc2->GetOKTrack()){estore->ClearData();continue;}

    bdc1x = bdc1->GetPosX() + prm->FindGeomPara("BDC1_Xoff"); //mm
    bdc1y = bdc1->GetPosY() + prm->FindGeomPara("BDC1_Yoff"); //mm
    bdc2x = bdc2->GetPosX() + prm->FindGeomPara("BDC2_Xoff"); //mm	 
    bdc2y = bdc2->GetPosY() + prm->FindGeomPara("BDC2_Yoff"); //mm	 
    tgtx = recobeam->GetBeamPosX();
    tgty = recobeam->GetBeamPosY();
    tgta = recobeam->GetBeamAngX();
    tgtb = recobeam->GetBeamAngY();
    fdc1x = fdc1->GetPosX() + prm->FindGeomPara("FDC1_Xoff"); //mm
    fdc1y = fdc1->GetPosY() + prm->FindGeomPara("FDC1_Yoff"); //mm
    fdc2x = fdc2->GetPosX() + prm->FindGeomPara("FDC2_Xoff"); //mm
    fdc2y = fdc2->GetPosY() + prm->FindGeomPara("FDC2_Yoff"); //mm
    fdc2a = fdc2->GetAngX() + prm->FindGeomPara("FDC2_DXDZoff"); //mm
    fdc2b = fdc2->GetAngY() + prm->FindGeomPara("FDC2_DYDZoff"); //mm
    Double_t dist_TGTFDC1 = prm->FindGeomPara("FDC1_Zoff")
      - prm->FindGeomPara("TGT_Zoff");
    fdc1a = (fdc1x-tgtx)/dist_TGTFDC1*1000.;//mrad
    fdc1b = (fdc1y-tgty)/dist_TGTFDC1*1000.;

    //-------------------------------------------------------------------
    tof713 = recobeam->GetTOF713();
    Double_t p0 = prm->FindRecoBeamPara("TOF713Brhomag0");//target position
    Double_t p1 = prm->FindRecoBeamPara("TOF713Brhomag1");//target position
    Double_t p2 = prm->FindRecoBeamPara("TOF713Brhomag2");//target position
    Double_t p3 = prm->FindRecoBeamPara("TOF713Brhomag3");//target position
    brho = p0 + p1*tof713 + p2*pow(tof713,2) + p3*pow(tof713,3);
    Double_t Ptot = TanaKinematics::Brho2TotalMome(brho,Zgate);

    double Etot = sqrt(Ptot*Ptot + M*M);

    TVector3 tgt_pos(tgtx,tgty,prm->FindGeomPara("TGT_Zoff"));//target position
    Double_t dxdz = fdc1a*0.001;//mrad->rad
    Double_t dydz = fdc1b*0.001;
    Double_t vec_u = sqrt( 1.0 + pow(dxdz,2) + pow(dydz,2) );
    TLorentzVector P(dxdz/vec_u * Ptot, 
		     dydz/vec_u * Ptot, 
		     1./vec_u      * Ptot, 
		     Etot);
    TBeamSimData particle(Z,A,P,tgt_pos);
    gBeamSimDataArray->push_back(particle);

    tree->Fill();
    estore->ClearData();
  }

  header->SetTitle(sheader.str().c_str());

  file->Write();
}
