#ifndef __CINT__
#include <TFile.h>
#include <TTree.h>
#include <TChain.h>
#include <TEntryList.h>
#include <TH1.h>
#include <TH2.h>
#include <TClonesArray.h>
#include <TLorentzVector.h>

#include "TArtStoreManager.hh"
#include "TArtCounter.hh"

#include "TanaParameterManager.hh"
#include "TanaRunParameters.hh"
#include "TanaRecoVertex.hh"
#include "TanaRecoMass.hh"
#include "TanaBeamInfo.hh"
#include "TanaHODPla.hh"
#include "TanaNeuLANDPla.hh"
#include "TanaNEBULAPla.hh"
#include "TanaPhysConst.hh"
#include "TanaNeutPla.hh"
#include "TanaParticle.hh"
#include "TanaCrosstalkAnalysis.hh"
#include "TanaCrosstalkAnalysis_NoKill.hh"
#include "TanaCrosstalkAnalysis_DiffSimple.hh"
#include "TanaCrosstalkAnalysis_Ver1.hh"
#include "TanaCrosstalkAnalysis_Ver2.hh"
#include "TanaKinematics.hh"
#include "TanaRecoBeamMINOSVertex.hh"
#include "TanaRecoFragMINOSVertex.hh"

#include "TRunSimParameter.hh"
#include "TFragSimParameter.hh"

#endif

void Analyze_1n()
{
  TFile *fin = new TFile("root/7lipn/outtree0000.root","readonly");

  TTree *tree = (TTree*)fin->Get("tree");
  TClonesArray *neutpla_array = new TClonesArray("TanaNeutPla");
  tree->SetBranchAddress("NeutPla",&neutpla_array);
  TClonesArray *vetopla_array = new TClonesArray("TanaNeutPla");
  tree->SetBranchAddress("VetoPla",&vetopla_array);

  TRunSimParameter *run_prm = (TRunSimParameter*)fin->Get("RunParameter");
  cout<<run_prm->fHeader<<endl;
  TFragSimParameter *frag_prm = (TFragSimParameter*)fin->Get("FragParameter");

  //---------------------------------------
  TString fout_name(fin->GetName());
  fout_name.ReplaceAll(".root","_analyzed.root");
  TFile *fout = new TFile(fout_name.Data(),"recreate");
  TH1* heff = new TH1I("heff","efficiency",2,-0.5,1.5);
  heff->SetMinimum(0);
  TH1* hidt = new TH2D("hidt","ID TOF",520,0.5,520.5,200,0,200);
  TH1* hidq = new TH2D("hidq","ID Q",520,0.5,520.5,200,0,200);
  TH1* hidx = new TH2D("hidx","ID X",520,0.5,520.5,200,-2000,2000);
  TH1* hidy = new TH2D("hidy","ID Y",520,0.5,520.5,200,-2000,2000);
  TH1* hidz = new TH2D("hidz","ID Z",520,0.5,520.5,200,6000,11000);
  TH1* hidrandx = new TH2D("hidrandx","ID X(rnd)",520,0.5,520.5,200,-2000,2000);
  TH1* hidrandy = new TH2D("hidrandy","ID Y(rnd)",520,0.5,520.5,200,-2000,2000);
  TH1* hidrandz = new TH2D("hidrandz","ID Z(rnd)",520,0.5,520.5,200,6000,11000);

  TH1* hkine = new TH1D("hkine","Kinetic energy ",200,200,300);
  TH1* hbeta = new TH1D("hbeta","beta",200,0.5,0.8);

  TH1* hcom = new TH1I("hcom","----- crosstalk analysis -----",1,0,1);
  TH1* hmulti = new TH1I("hmulti","Multi",10,0.5,10.5);

  TH1* hid1id2 = new TH2D("hid1id2","ID1 ID2",520,0.5,520.5,520,0.5,520.5);

  TH1* hdtdx_s = new TH2D("hdtdx_s","dt dx SAME",200,0,20, 200,0,2000);
  TH1* hdtdx_s_neu = new TH2D("hdtdx_s_neu","dt dx SAME NeuLAND",200,0,20, 200,0,2000);
  TH1* hdtdx_s_neb = new TH2D("hdtdx_s_neb","dt dx SAME NEBULA",200,0,20, 200,0,2000);
  TH1* hb12q1_s = new TH2D("hb12q1_s","1/Beta12 Q1 SAME",200,-10,10, 200,0,200);
  TH1* hb12q1_s_neu = new TH2D("hb12q1_s_neu","1/Beta12 Q1 SAME NeuLAND",200,-10,10, 200,0,200);
  TH1* hb12q1_s_neb = new TH2D("hb12q1_s_neb","1/Beta12 Q1 SAME NEBULA",200,-10,10, 200,0,200);
  TH1* hb12q2_s = new TH2D("hb12q2_s","1/Beta12 Q2 SAME",200,-10,10, 200,0,200);

  TH1* hdtdx_d = new TH2D("hdtdx_d","dt dx DIFF",200,0,20, 200,0,4000);
  TH1* hb12q1_d = new TH2D("hb12q1_d","1/Beta12 Q1 DIFF",200,-10,10, 200,0,200);
  TH1* hb12q2_d = new TH2D("hb12q2_d","1/Beta12 Q2 DIFF",200,-10,10, 200,0,200);
  TH1* hbrq1_d = new TH2D("hbrq1_d","Beta01/Beta12 Q1 DIFF",200,-10,10, 200,0,200);
  TH1* hbrq2_d = new TH2D("hbrq2_d","Beta01/Beta12 Q2 DIFF",200,-10,10, 200,0,200);
  //---------------------------------------

  TanaCrosstalkAnalysis *cta = new TanaCrosstalkAnalysis_Ver2;
  cta->SkipLoad();
  cta->SkipSubtractTargetTOF();
  cta->SetNeutThreshold(6);
  cta->SetVetoThreshold(1);

  cta->SkipKillCrosstalk();

  Long64_t nentries = tree->GetEntries();
  TArtCounter *counter = TArtCounter::SetCounter(nentries);
  for (Long64_t ieve=0;ieve<nentries;++ieve){
    counter->Draw();

    cta->Clear();
    neutpla_array->Delete();
    vetopla_array->Delete();

    tree->GetEntry(ieve);


    cta->SetNeutPlaArray(neutpla_array);
    cta->SetVetoPlaArray(vetopla_array);
    cta->SetTargetPosZ(frag_prm->fTargetPosition.Z());
    cta->Analyze();

    Int_t npla = neutpla_array->GetEntries();
    hmulti->Fill(npla);
    for (Int_t ipla=0;ipla<npla;++ipla){
      TanaNeutPla *pla = (TanaNeutPla*)neutpla_array->At(ipla);

      hidt->Fill(pla->GetID(), pla->GetTOF());
      hidq->Fill(pla->GetID(), pla->GetQ());
      hidx->Fill(pla->GetID(), pla->GetX());
      hidy->Fill(pla->GetID(), pla->GetY());
      hidz->Fill(pla->GetID(), pla->GetZ());
      hidrandx->Fill(pla->GetID(), pla->GetRandX());
      hidrandy->Fill(pla->GetID(), pla->GetRandY());
      hidrandz->Fill(pla->GetID(), pla->GetRandZ());

    }

    if (npla>0){
      TClonesArray *p_array = cta->GetParticleArray();
      TanaParticle *neut = (TanaParticle*)p_array->At(0);
      hbeta->Fill(neut->GetBeta());
      hkine->Fill(neut->GetT());
    }

    Double_t binv12 = cta->GetBinv12();
    Double_t beta01 = cta->GetBeta01();
    hid1id2->Fill(cta->GetID1(), cta->GetID2());
    hid1id2->Fill(cta->GetID1(), cta->GetID2());
    if (cta->GetDLayer()==0){//SAME
      hdtdx_s->Fill(cta->GetDt(), cta->GetDPos());
      hb12q1_s->Fill(cta->GetBinv12(), cta->GetQ1());
      hb12q2_s->Fill(cta->GetBinv12(), cta->GetQ2());

      if (cta->GetLayer(0)==1){//NeuLAND
	hdtdx_s_neu->Fill(cta->GetDt(), cta->GetDPos());
	hb12q1_s_neu->Fill(cta->GetBinv12(), cta->GetQ1());
      }else {// NEBULA
	hdtdx_s_neb->Fill(cta->GetDt(), cta->GetDPos());
	hb12q1_s_neb->Fill(cta->GetBinv12(), cta->GetQ1());
      }


    }else{// DIFF
      hdtdx_d->Fill(cta->GetDt(), cta->GetDPos());
      hb12q1_d->Fill(cta->GetBinv12(), cta->GetQ1());
      hb12q2_d->Fill(cta->GetBinv12(), cta->GetQ2());
      hbrq1_d->Fill(beta01*binv12, cta->GetQ1());
      hbrq2_d->Fill(beta01*binv12, cta->GetQ2());
    }


    if (npla>0) heff->Fill(1);
    else        heff->Fill(0);

  }

  fout->Write();

}
