#ifndef __CINT__
#include <TFile.h>
#include <TTree.h>
#include <TChain.h>
#include <TEntryList.h>
#include <TH1.h>
#include <TH2.h>
#include <TClonesArray.h>
#include <TLorentzVector.h>

#include "TArtStoreManager.hh"
#include "TArtCounter.hh"

#include "TanaParameterManager.hh"
#include "TanaRunParameters.hh"
#include "TanaRecoVertex.hh"
#include "TanaRecoMass.hh"
#include "TanaBeamInfo.hh"
#include "TanaHODPla.hh"
#include "TanaPhysConst.hh"
#include "TanaNeutPla.hh"
#include "TanaParticle.hh"
#include "TanaCrosstalkAnalysis.hh"
#include "TanaCrosstalkAnalysis_NoKill.hh"
#include "TanaCrosstalkAnalysis_DiffSimple.hh"
#include "TanaCrosstalkAnalysis_Ver1.hh"
#include "TanaCrosstalkAnalysis_Ver2.hh"
#include "TanaKinematics.hh"
#include "TanaRecoBeamMINOSVertex.hh"
#include "TanaRecoFragMINOSVertex.hh"

#include "TRunSimParameter.hh"
#include "TFragSimParameter.hh"

#include "TanaNeuLANDPla.hh"
#include "TArtNEBULAPla.hh"

#endif

void Analyze_1n_TArt()
{
  TFile *fin = new TFile("root/7lipn/outtree0000.root","readonly");

  TTree *tree = (TTree*)fin->Get("tree");
  TClonesArray *nebula_array = new TClonesArray("TArtNEBULAPla");
  tree->SetBranchAddress("NEBULAPla",&nebula_array);

  TRunSimParameter *run_prm = (TRunSimParameter*)fin->Get("RunParameter");
  cout<<run_prm->fHeader<<endl;
  TFragSimParameter *frag_prm = (TFragSimParameter*)fin->Get("FragParameter");

  //---------------------------------------
  TString fout_name(fin->GetName());
  fout_name.ReplaceAll(".root","_analyzed.root");
  TFile *fout = new TFile(fout_name.Data(),"recreate");
  TH1* heff = new TH1I("heff","efficiency",2,-0.5,1.5);
  heff->SetMinimum(0);
  TH1* hmulti = new TH1I("hmulti","multi",10,0.5,10.5);
  TH1* hidt = new TH2D("hidt","ID TOF",520,0.5,520.5,200,0,200);
  TH1* hidq = new TH2D("hidq","ID Q",520,0.5,520.5,200,0,200);
  TH1* hidx = new TH2D("hidx","ID X",520,0.5,520.5,200,-2000,2000);
  TH1* hidy = new TH2D("hidy","ID Y",520,0.5,520.5,200,-2000,2000);
  TH1* hidz = new TH2D("hidz","ID Z",520,0.5,520.5,200,6000,11000);
  //---------------------------------------

  Long64_t nentries = tree->GetEntries();
  TArtCounter *counter = TArtCounter::SetCounter(nentries);
  for (Long64_t ieve=0;ieve<nentries;++ieve){
    counter->Draw();

    nebula_array->Delete();

    tree->GetEntry(ieve);

    Int_t npla = nebula_array->GetEntries();
    hmulti->Fill(npla);
    for (Int_t ipla=0;ipla<npla;++ipla){
      TArtNEBULAPla *pla = (TArtNEBULAPla*)nebula_array->At(ipla);

      if (pla->GetQAveCal()<6) continue;
      hidt->Fill(pla->GetID(), pla->GetTAveSlw());
      hidq->Fill(pla->GetID(), pla->GetQAveCal());
      hidx->Fill(pla->GetID(), pla->GetPos(0));
      hidy->Fill(pla->GetID(), pla->GetPos(1));
      hidz->Fill(pla->GetID(), pla->GetPos(2));
    }

    if (npla>0) heff->Fill(1);
    else        heff->Fill(0);

  }

  fout->Write();

}
