/*
  Simple analysis example
  root[0] .L macros/examples/analysis_example.cc+g
  root[1] analysis_example()
*/
#include <iostream>
#include <sstream>

#include "TFile.h"
#include "TSystem.h"
#include "TTree.h"
#include "TString.h"
#include "TH1.h"
#include "TH2.h"
#include "TClonesArray.h"
#include "TRandom3.h"

#include "TBeamSimData.hh"
#include "TFragSimData.hh"
#include "TNEBULASimData.hh"
#include "TArtNEBULAPla.hh"
#include "TArtCrosstalkAnalysis_NEBULANeuLAND_samurai21_7.hh"
#include "TArtCrosstalkAnalysis_NEBULANeuLAND_samurai21_8.hh"

#include "TRunSimParameter.hh"
#include "TFragSimParameter.hh"


#include "TArtCounter.hh"

void Analyze_1n(TString filename ="samurai21_7_1n_0000.root")
{
  std::stringstream ss;
  ss<<"root/storage/"<<filename.Data();

  TFile *file = new TFile(ss.str().c_str(),"readonly");

  TRunSimParameter *RunPrm = 0;
  file->GetObject("RunParameter",RunPrm);
  RunPrm->Print();

  TFragSimParameter *FragPrm = 0;
  file->GetObject("FragParameter",FragPrm);
  FragPrm->Print();


  TTree *tree = 0;
  file->GetObject(RunPrm->fTreeName.Data(),tree);

  TClonesArray *NEBULAPlaArray = new TClonesArray("TArtNEBULAPla",144);

  tree->SetBranchAddress("beam",&gBeamSimDataArray);
  tree->SetBranchAddress("NEBULAPla",&NEBULAPlaArray);

  stringstream ss_out;
  ss_out<<"root/analyzed_"<<filename.Data();
  TFile *outfile = new TFile(ss_out.str().c_str(),"recreate");
  
  TH1* hidlayer = new TH2D("hidlayer","ID Layer",606,0.5,606.5, 3,0.5,3.5);
  TH1* hidq = new TH2D("hidq","ID Q",606,0.5,606.5, 150,0,150);
  TH1* hidx = new TH2D("hidx","ID X",606,0.5,606.5, 100,-2000,2000);
  TH1* hidy = new TH2D("hidy","ID Y",606,0.5,606.5, 100,-2000,2000);
  TH1* hidz = new TH2D("hidz","ID Z",606,0.5,606.5, 100,10000,20000);
  TH1* hxy = new TH2D("hxy","XY",100,-2000,2000, 100,-2000,2000);
  TH1* hzy = new TH2D("hzy","ZY",100,10000,20000, 100,-2000,2000);

  TH1* hbeamposxy = new TH2D("hbeamposxy","Beam Pos XY (mm)",200,-50,50,200,-50,50);
  TH1* hmulti = new TH1I("hmulti","NEBULA Multiplicity",10,-0.5,9.5);
  TH1* hmulti_cta = new TH1I("hmulti_cta","NEBULA Multiplicity cta",10,-0.5,9.5);
  TH1* htht = new TH1D("htht","Theta (rad)",50,0,0.3);

  TH1* hb0112_d = new TH1D("hb0112_d","Beta01/Beta12 DIFF",400,-4,4);
  TH1* hb0112q1_d = new TH2D("hb0112q1_d","Beta01/Beta12 Q1 DIFF",400,-4,4,100,0,100);
  TH1* hb0112q1_d12 = new TH2D("hb0112q1_d12","Beta01/Beta12 Q1 DIFF12",400,-4,4,100,0,100);
  TH1* hb0112q1_d13 = new TH2D("hb0112q1_d13","Beta01/Beta12 Q1 DIFF13",400,-4,4,100,0,100);
  TH1* hb0112q1_d23 = new TH2D("hb0112q1_d23","Beta01/Beta12 Q1 DIFF23",400,-4,4,100,0,100);

  TH1* hb12invq2_d12 = new TH2D("hb12invq2_d12","Binv12 Q2 DIFF12",400,-4,4,100,0,100);
  TH1* hb12invq2_d13 = new TH2D("hb12invq2_d13","Binv12 Q2 DIFF13",400,-4,4,100,0,100);
  TH1* hb12invq2_d23 = new TH2D("hb12invq2_d23","Binv12 Q2 DIFF23",400,-4,4,100,0,100);

  TH1* hb0112q1_s = new TH2D("hb0112q1_s","Beta01/Beta12 Q1 SAME",200,-2,2,100,0,100);
  TH1* hb0112_s = new TH1D("hb0112_s","Beta01/Beta12 SAME",200,-4,4);

  TArtCrosstalkAnalysis *ctana=0;

  if (filename.Contains("samurai21_7")){
    ctana = new TArtCrosstalkAnalysis_NEBULANeuLAND_samurai21_7;
  }else if (filename.Contains("samurai21_8")){
    ctana = new TArtCrosstalkAnalysis_NEBULANeuLAND_samurai21_8;
  }else{
    cout<<"unknown settting"<<endl;
    return;
  }
  ctana->SetNEBULAPlaArray(NEBULAPlaArray);
  ctana->SetNeutThreshold(6);
  ctana->SetVetoThreshold(1);
  //ctana->SetDoKillCrosstalk(false);
  //ctana->SetDoUseVeto(false);

  cout<<ctana->GetName()<<endl;
  cout<<"DoKillCrosstalk="<<ctana->GetDoKillCrosstalk()
      <<endl;

  Int_t neve = tree->GetEntries();
  TArtCounter *counter = TArtCounter::SetCounter(neve);
  for (Int_t ieve=0;ieve<neve;++ieve){
    counter->Draw();

    gBeamSimDataArray->clear();
    NEBULAPlaArray->Delete();

    tree->GetEntry(ieve);

    // Get beam data
    TBeamSimData beam_24o;
    Int_t nbeam = gBeamSimDataArray->size();
    for (Int_t ibeam=0;ibeam<nbeam;++ibeam){
      TBeamSimData beam = (*gBeamSimDataArray)[ibeam];
      hbeamposxy->Fill(beam.fPosition.x(), beam.fPosition.y());
      if (ibeam==0) beam_24o = beam;
    }

    // Get data
    Int_t npla = NEBULAPlaArray->GetEntries();
    hmulti->Fill(npla);

    // add TGT-MAG
    for (Int_t ipla=0;ipla<npla;++ipla){
      TArtNEBULAPla *pla = (TArtNEBULAPla*)NEBULAPlaArray->At(ipla);
      Double_t z_new = pla->GetPos(2) - FragPrm->fTargetPosition.z();
      pla->SetPos(z_new,2);
    }

    // cross talk analysis
    ctana->Analyze();

    Int_t npla_cta = NEBULAPlaArray->GetEntries();
    hmulti_cta->Fill(npla_cta);

    for (Int_t ipla=0;ipla<npla_cta;++ipla){
      TArtNEBULAPla *pla = (TArtNEBULAPla*)NEBULAPlaArray->At(ipla);
      hidlayer->Fill(pla->GetID(), pla->GetLayer());
      hidq->Fill(pla->GetID(), pla->GetQAveCal());
      hidx->Fill(pla->GetID(), pla->GetPos(0));
      hidy->Fill(pla->GetID(), pla->GetPos(1));
      hidz->Fill(pla->GetID(), pla->GetPos(2));
      hxy->Fill(pla->GetPos(0), pla->GetPos(1));
      hzy->Fill(pla->GetPos(2), pla->GetPos(1));

      Double_t tht = sqrt(pow(pla->GetPos(0),2)+pow(pla->GetPos(1),2))
	/ pla->GetPos(2);
      htht->Fill(tht);
    }

    if (npla_cta>1){
      if (ctana->GetDLayer()!=0){// DIFF
	  hb0112_d->Fill(ctana->GetBeta01()*ctana->GetBinv12());
	  hb0112q1_d->Fill(ctana->GetBeta01()*ctana->GetBinv12(),ctana->GetQ1());

	  if ( (ctana->GetLayer1()==1 && ctana->GetLayer2()==2) ||
	       (ctana->GetLayer1()==2 && ctana->GetLayer2()==1)   ){
	    hb0112q1_d12->Fill(ctana->GetBeta01()*ctana->GetBinv12(),ctana->GetQ1());
	    hb12invq2_d12->Fill(ctana->GetBinv12(),ctana->GetQ2());
	  }

	  if ( (ctana->GetLayer1()==1 && ctana->GetLayer2()==3) ||
	       (ctana->GetLayer1()==3 && ctana->GetLayer2()==1)   ){
	    hb0112q1_d13->Fill(ctana->GetBeta01()*ctana->GetBinv12(),ctana->GetQ1());
	    hb12invq2_d13->Fill(ctana->GetBinv12(),ctana->GetQ2());
	  }
	  if ( (ctana->GetLayer1()==2 && ctana->GetLayer2()==3) ||
	       (ctana->GetLayer1()==3 && ctana->GetLayer2()==2)   ){
	    hb0112q1_d23->Fill(ctana->GetBeta01()*ctana->GetBinv12(),ctana->GetQ1());
	    hb12invq2_d23->Fill(ctana->GetBinv12(),ctana->GetQ2());
	  }
      }else{ // SAME
	  hb0112_s->Fill(ctana->GetBeta01()*ctana->GetBinv12());
	  hb0112q1_s->Fill(ctana->GetBeta01()*ctana->GetBinv12(),ctana->GetQ1());
      }

    }

  }
  outfile->Write();
  outfile->Close();
}
