/*
  Simple analysis example
  root[0] .L macros/examples/analysis_crosstalk_example.cc+g
  root[1] analysis_crosstalk_example()
*/
#include <iostream>

#include "TFile.h"
#include "TTree.h"
#include "TH1.h"
#include "TH2.h"
#include "TClonesArray.h"
#include "TRandom3.h"

#include "TBeamSimData.hh"
#include "TFragSimData.hh"
#include "TNEBULASimData.hh"
#include "TNeuLANDSimData.hh"
#include "TArtNEBULAPla.hh"

#include "TRunSimParameter.hh"
#include "TFragSimParameter.hh"

#include "TArtCrosstalkAnalysis.hh"
#include "TArtCrosstalkAnalysis_DoNothing.hh"
#include "TArtCrosstalkAnalysis_NEBULANeuLAND_Example.hh"

Double_t CalcErel(TLorentzVector P24o, TClonesArray *PlaArray, Int_t Nneut=3);

void ana_27o(TString filename="root/storage/samurai21_2_27o_seq_0000.root",
	     Bool_t DoKillCrosstalk=true)
{

  Double_t TanTheta_duct = TMath::Pi();
  if (filename.Contains("samurai21_")) {
    TanTheta_duct = 5.94 *TMath::Pi()/180.;// duct acceptance in deg.

    if (filename.Contains("samurai21_6_"))
      TanTheta_duct = 9.85 *TMath::Pi()/180.;// duct acceptance in deg. for setup6
    cout<<"TanTheta_duct= "<<TanTheta_duct<<" rad"<<endl;
  }

  // resolutions
  //                              dE diff at TGT,    reso at SAMURAI, E straggling
  const Double_t Preso_rel = sqrt( pow(0.000954,2) + pow(1./800.,2) + pow(0.0005,2) );
  const Double_t Angreso = 0.00173;// Coul. Mul. Scat. of 24O at 1.6g/cm2 (rad)

  TFile *file = new TFile(filename.Data(),"readonly");

  TRunSimParameter *RunPrm = 0;
  file->GetObject("RunParameter",RunPrm);
  RunPrm->Print();

  TFragSimParameter *FragPrm = 0;
  file->GetObject("FragParameter",FragPrm);
  FragPrm->Print();

  TTree *tree = 0;
  file->GetObject(RunPrm->fTreeName.Data(),tree);

  TClonesArray *NEBULAPlaArray = new TClonesArray("TArtNEBULAPla",605);

  tree->SetBranchAddress("beam",&gBeamSimDataArray);
  tree->SetBranchAddress("fragment",&gFragSimDataArray);
  tree->SetBranchAddress("NEBULAPla",&NEBULAPlaArray);

  TArtCrosstalkAnalysis *ctana = new TArtCrosstalkAnalysis_NEBULANeuLAND_Example;
  ctana->SetNEBULAPlaArray(NEBULAPlaArray);
  ctana->SetNeutThreshold(6);
  ctana->SetVetoThreshold(1);
  ctana->SetDoKillCrosstalk(DoKillCrosstalk);

  std::cout<<std::endl
	   <<"DoKillCorsstalk = "<<DoKillCrosstalk
	   <<std::endl;

  //-----------------------------------
  TString outfilename = filename;
  outfilename.ReplaceAll("storage/","");
  outfilename.ReplaceAll("samurai21","analyzed_samurai21");
  if (!DoKillCrosstalk)  outfilename.ReplaceAll("analyzed_samurai21","analyzed_nocut_samurai21");
//  TFile *fileout = new TFile(outfilename.Data(),"recreate");
  TFile *fileout = new TFile("root/temp.root","recreate");
  cout<<"out file name = "<<fileout->GetName()<<endl;
  fileout->cd();

  TH1* hbeamposxy = new TH2D("hbeamposxy","Beam Pos XY (mm)",200,-50,50,200,-50,50);
  TH1* hmulti     = new TH1I("hmulti","NEBULA Multiplicity",30,-0.5,29.5);
  TH1* hmulti_cta = new TH1I("hmulti_cta","NEBULA Multiplicity cta",30,-0.5,29.5);
  TH1* hidq = new TH2D("hidq","ID Q",605,0.5,605.5, 200,0,200);
  TH1* hidq_cta = new TH2D("hidq_cta","ID Q Thre,VETO",605,0.5,605.5, 200,0,200);
  TH1* hidx = new TH2D("hidx","ID X(mm)",605,0.5,605.5, 200,-2000,2000);
  TH1* hidy = new TH2D("hidy","ID Y(mm)",605,0.5,605.5, 200,-2000,2000);
  TH1* hidz = new TH2D("hidz","ID Z(mm)",605,0.5,605.5, 200,10000,20000);
  TH1* hnebulatq = new TH2D("hnebulatq","NEBULA TOF(ns) Q(MeVee)",200,0,200,200,0,200);
  TH1* hnebulaxy = new TH2D("hnebulaxy","NEBULA Pos XY (mm)",200,-2000,2000,200,-2000,2000);
  TH1* hnebulazx = new TH2D("hnebulazx","NEBULA Pos ZX (mm)",200,10000,20000,200,-2000,2000);
  TH1* hnebulazy = new TH2D("hnebulazy","NEBULA Pos ZY (mm)",200,10000,20000,200,-2000,2000);

  TH1* hthtxthty = new TH2D("hthtxthty","NEBULA Theta_x Theta_y (mrad)",100,-200,200, 100,-200,200);
  TH1* hthtxthty_cta = new TH2D("hthtxthty_cta","NEBULA Theta_x Theta_y (mrad) cta",100,-200,200, 100,-200,200);

  TH1* hid1id2     = new TH2I("hid1id2","ID1 ID2",605,0.5,605.5, 605,0.5,605.5);
  TH1* hdtdr_s     = new TH2D("hdtdr_s","dt dr SAME",200,0,10, 200,0,1000);
  TH1* hb0112q2_s     = new TH2D("hb0112q2_s","beta01/beta12 Q2 SAME",200,-4,4, 200,0,150);
  TH1* hb0112q2_s_neuland = new TH2D("hb0112q2_s_neuland","beta01/beta12 Q2 SAME NeuLAND",200,-4,4, 200,0,150);
  TH1* hb0112q2_s_nebula  = new TH2D("hb0112q2_s_nebula","beta01/beta12 Q2 SAME NEBULA",200,-4,4, 200,0,150);

  TH1* hb12invq1_d     = new TH2D("hb12invq1_d","binv12 Q1 DIFF",200,-6.,6., 200,0,150);
  TH1* hb12invq2_d     = new TH2D("hb12invq2_d","binv12 Q2 DIFF",200,-6.,6., 200,0,150);
  TH1* hb0112q1_d     = new TH2D("hb0112q1_d","beta01/beta12 Q1 DIFF",200,-4,4, 200,0,150);

  TH1* herel     = new TH1D("herel","Erel (MeV)",200,0,10);
  TH1* herel_duct = new TH1D("herel_duct","Erel dcutOK(MeV)",200,0,10);
  TH1* herel_r     = new TH1D("herel_r","Erel reso (MeV)",200,0,10);

  TH1* hokduct = new TH1I("hokduct","Duct Acceptance 1:OK 0:NG",2,-0.5,1.5);
  hokduct->SetMinimum(0);

  //-----------------------------------
  Int_t neve = tree->GetEntries();
  for (Int_t ieve=0;ieve<neve;++ieve){

    if (ieve%1000==0){
      cout<<"\r events: "<<ieve<<" / "<<neve
	  <<" ("<<100.*ieve/neve<<"%)"
	  <<flush;
    }

    gBeamSimDataArray->clear();
    gFragSimDataArray->clear();
    NEBULAPlaArray->Delete();

    tree->GetEntry(ieve);

    //-----------------------------------
    // Get beam data
    Int_t nbeam = gBeamSimDataArray->size();
    for (Int_t ibeam=0;ibeam<nbeam;++ibeam){
      TBeamSimData beam = (*gBeamSimDataArray)[ibeam];
      hbeamposxy->Fill(beam.fPosition.fX, beam.fPosition.fY);
    }
    // check duct acceptance
    Int_t OK_duct=1;
    for (Int_t ibeam=0;ibeam<nbeam;++ibeam){
      TBeamSimData beam = (*gBeamSimDataArray)[ibeam];
      Double_t x_tmp = beam.fMomentum.Px();
      Double_t y_tmp = beam.fMomentum.Py();
      Double_t z_tmp = beam.fMomentum.Pz();
      Double_t TanTheta = sqrt(x_tmp*x_tmp+y_tmp*y_tmp)/z_tmp;
      if (TanTheta>TanTheta_duct) OK_duct = 0;
    }
    hokduct->Fill(OK_duct);

    //if (OK_duct==0) continue;

    //-----------------------------------
    // get fragment data
    Bool_t OK_FDC1    = false;
    Bool_t OK_WinHole = false;
    Bool_t OK_FDC2    = false;
    Bool_t OK_HOD     = false;
    Int_t nfrag = gFragSimDataArray->size();
    for (Int_t ifrag=0;ifrag<nfrag;++ifrag){
      TFragSimData frag = (*gFragSimDataArray)[ifrag];
      if (frag.fDetectorName == "FDC1") OK_FDC1 = true;
      if (frag.fDetectorName == "WindowHole") OK_WinHole = true;
      if (frag.fDetectorName == "FDC2") OK_FDC2 = true;
      if (frag.fDetectorName == "HOD") OK_HOD = true;
    }

    //-----------------------------------
    // Get NEBULA data
    Int_t npla = NEBULAPlaArray->GetEntries();
    hmulti->Fill(npla);
    for (Int_t ipla=0;ipla<npla;++ipla){
      TArtNEBULAPla *pla = (TArtNEBULAPla*)NEBULAPlaArray->At(ipla);

      //add dist. between TGT-Magnet
      Double_t z_new = pla->GetPos(2) - FragPrm->fTargetPosition.z();
      pla->SetPos(z_new,2);

      hidq->Fill(pla->GetID(), pla->GetQAveCal());
      hidx->Fill(pla->GetID(), pla->GetPos(0));
      hidy->Fill(pla->GetID(), pla->GetPos(1));
      hidz->Fill(pla->GetID(), pla->GetPos(2));

      hnebulatq->Fill(pla->GetTAveSlw(),pla->GetQAveCal());
      hnebulaxy->Fill(pla->GetPos(0),pla->GetPos(1));
      hnebulazx->Fill(pla->GetPos(2),pla->GetPos(0));
      hnebulazy->Fill(pla->GetPos(2),pla->GetPos(1));

      Double_t Theta_x = pla->GetPos(0)/pla->GetPos(2)*1000.;
      Double_t Theta_y = pla->GetPos(1)/pla->GetPos(2)*1000.;
      hthtxthty->Fill(Theta_x, Theta_y);
    }

    //-----------------------------------
    // crosstalk cut
    ctana->Analyze();
    Int_t multi_cta = NEBULAPlaArray->GetEntries();
    hmulti_cta->Fill(multi_cta);

    hid1id2->Fill(ctana->GetID1(), ctana->GetID2());
    if (ctana->GetDLayer()==0){// SAME Wall 
      hdtdr_s->Fill(ctana->GetDt(), ctana->GetDPos());
      hb0112q2_s->Fill(ctana->GetBeta01()*ctana->GetBinv12(),ctana->GetQ2());
      if (ctana->GetID1()<200) hb0112q2_s_nebula->Fill(ctana->GetBeta01()*ctana->GetBinv12(),ctana->GetQ2());
      else                    hb0112q2_s_neuland->Fill(ctana->GetBeta01()*ctana->GetBinv12(),ctana->GetQ2());
    }else{                     // DIFF wall
      hb12invq1_d->Fill(ctana->GetBinv12(), ctana->GetQ1());
      hb12invq2_d->Fill(ctana->GetBinv12(), ctana->GetQ2());
      hb0112q1_d->Fill(ctana->GetBeta01()*ctana->GetBinv12(), ctana->GetQ1());
    }

    TLorentzVector P24o;
    // momentum of 24O
    for (Int_t ibeam=0;ibeam<nbeam;++ibeam){
      TBeamSimData beam = (*gBeamSimDataArray)[ibeam];
      if (beam.fZ==8) P24o = beam.fMomentum;
    }
    Double_t Erel = CalcErel(P24o,NEBULAPlaArray);
    herel->Fill(Erel);
    if (OK_duct==1) herel_duct->Fill(Erel);

    // resolution
    TLorentzVector P24o_r = P24o;
    Double_t dthtx = gRandom->Gaus(0,Angreso);
    Double_t dthty = gRandom->Gaus(0,Angreso);
    P24o_r.RotateY(dthtx);
    P24o_r.RotateX(dthty);
    Double_t p_old = P24o_r.P();
    Double_t p_new = gRandom->Gaus(p_old,p_old*Preso_rel);
    P24o_r.SetPxPyPzE(P24o.Px() * p_new/p_old,
		      P24o.Py() * p_new/p_old,
		      P24o.Pz() * p_new/p_old,
		      sqrt(pow(p_new,2)+pow(P24o.M(),2)) );

    Double_t Erel_r = CalcErel(P24o_r,NEBULAPlaArray);
    herel_r->Fill(Erel_r);

    for (Int_t ipla=0;ipla<multi_cta;++ipla){
      TArtNEBULAPla *pla = (TArtNEBULAPla*)NEBULAPlaArray->At(ipla);
      Double_t Theta_x = pla->GetPos(0)/pla->GetPos(2)*1000.;
      Double_t Theta_y = pla->GetPos(1)/pla->GetPos(2)*1000.;
      hthtxthty_cta->Fill(Theta_x, Theta_y);
    }

    
    //-----------------------------------

  }//for (Int_t ieve=0;ieve<neve;++ieve){
  fileout->Write();
  fileout->Close();

}
//________________________________________________________
Double_t CalcErel(TLorentzVector P24o, TClonesArray *PlaArray, Int_t Nneut){

  if (PlaArray->GetEntries()<Nneut) return -1000;

  // momentum of 24O
  std::vector<TLorentzVector> P(Nneut+1);

  P[0] = P24o;
  
  // momentum of neutrons
  for (int i=0;i<Nneut;++i){
    TArtNEBULAPla *pla = (TArtNEBULAPla*)PlaArray->At(i);
    Double_t t = pla->GetTAveSlw();
    Double_t x = pla->GetPos(0);
    Double_t y = pla->GetPos(1);
    Double_t z = pla->GetPos(2);
    Double_t FL = sqrt( x*x + y*y + z*z );
    Double_t c_light = 299.792458; // mm/ns
    Double_t beta = FL/t/c_light;
    Double_t gamma = 1./sqrt((1.0+beta)*(1.0-beta));
    Double_t Mn = 939.565;      // MeV
    Double_t Pabs = Mn*beta*gamma;
    
    P[i+1].SetPxPyPzE(Pabs*x/FL,
		      Pabs*y/FL,
		      Pabs*z/FL,
		      Mn*gamma);
  }
  Double_t Mtot=0;
  TLorentzVector Ptot(0,0,0,0);
  for (int i=0;i<Nneut+1;++i){
    Mtot += P[i].M();
    Ptot += P[i];
  }
      
  Double_t Erel = Ptot.M() - Mtot;

  return Erel;
}
//________________________________________________________
